<?php

namespace Drupal\link_as_popup\Plugin\Field\FieldFormatter;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\link\LinkItemInterface;

/**

 * Plugin implementation of the 'link' to convert into Link formatter.
 *
 * @FieldFormatter(
 *   id = "link_as_popup",
 *   label = @Translation("Link As Popup | Chromeless/Modal"),
 *   field_types = {
 *     "link",
 *   }
 * )
 */
class LinkAsPopupFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $settings = $this->getSettings();
    foreach ($items as $delta => $item) {
      $url = $this->buildUrl($item);
      $options = $url->getOptions();
      $url_string = $url->toString();
        $title = $item->title ?: $url_string;
        // Create the render array.
        $elements[$delta] = [
          '#type' => 'link',
          '#title' => $title,
          '#url' => $url,
          '#attributes' => [
              'class' => ['link-as-popup'],
           ],
          '#attached' => [
            'library'=>['link_as_popup/popup']
          ],
       ];
       if (isset($options['attributes']) && isset($options['attributes']['data-trigger'])) {
        switch ($options['attributes']['data-trigger']){
          case 'link_as_popop_chromeless':
            $elements[$delta]['#attributes']['class'][] = 'popup-as-chromeless';
        break;
           case 'link_as_popop_dialog':
             $elements[$delta]['#attributes']['class'][] = 'popup-as-dialog';
        break;
         case 'link_as_popop_drupal_ajax':
            $elements[$delta]['#attached']['library'][] = 'core/drupal.dialog.ajax';
            $elements[$delta]['#attributes']['class'][] = 'popup-as-drupal-ajax';
            $elements[$delta]['#attributes']['class'][] = 'use-ajax';
            $elements[$delta]['#attributes']['data-dialog-type'] = 'modal';
            $dialogOptions = [
              'position' => ['my' => 'center center', 'at' => 'center center'],
              'draggable' => TRUE,
              'resizable' => TRUE,
             ];
             if (!empty($options['attributes']['data-height']) ){
              $dialogOptions['height'] = $options['attributes']['data-height'];
             }
             if (!empty($options['attributes']['data-width']) ){
              $dialogOptions['width'] = $options['attributes']['data-width'];
             }
            $elements[$delta]['#attributes']['data-dialog-options'] = Json::encode($dialogOptions);
        break;
        }
      }
    }
    return $elements;
  }

  /**
   * Builds the \Drupal\Core\Url object for a link field item.
   *
   * @param \Drupal\link\LinkItemInterface $item
   *   The link field item being rendered.
   *
   * @return \Drupal\Core\Url
   *   A Url object.
   */
  protected function buildUrl(LinkItemInterface $item) {
    $url = $item->getUrl() ?: Url::fromRoute('<none>');
    $settings = $this->getSettings();
    $options = $item->options;
    $options += $url->getOptions();

    // Add optional 'rel' attribute to link options.
    if (!empty($settings['rel'])) {
      $options['attributes']['rel'] = $settings['rel'];
    }

    $active = false;
    if ( (isset($options['attributes']) && isset($options['attributes']['target'])) && $options['attributes']['target']==='dialog'){
      $options['attributes']['data-trigger'] = 'link_as_popop_dialog';
      $active = true;
    }
    if ( (isset($options['attributes']) && isset($options['attributes']['target'])) && $options['attributes']['target']==='chromeless'){
      $options['attributes']['data-trigger'] = 'link_as_popop_chromeless';
      $active = true;
    }
    if ( (isset($options['attributes']) && isset($options['attributes']['target'])) && $options['attributes']['target']==='drupal_ajax'){
      $options['attributes']['data-trigger'] = 'link_as_popop_drupal_ajax';
      $active = true;
    }
    // Remove size values/attribues if not used.  Also, reset target '_blank' action for js degrade
    if ($active===false){
      unset($options['attributes']['data-width']);
      unset($options['attributes']['data-height']);
    } else {
     $id = 'popup-as-'. Html::getUniqueId(md5($url->toString()));
      $options['attributes']['target']='_blank';
      $options['attributes']['id']=$id;
      // Ensure data-title is set
      if (!isset($options['attributes']['data-title']) || $options['attributes']['data-title']===NULL){
        $options['attributes']['data-title'] = $item->title;
      }
    }

    $url->setOptions($options);
    return $url;
  }

}
