<?php

namespace Drupal\link_as_popup\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\link\Plugin\Field\FieldWidget\LinkWidget;

/**
 * Plugin implementation of the 'link_as_popup_field_widget' widget.
 *
 * @FieldWidget(
 *   id = "link_as_popup_field_widget",
 *   label = @Translation("Link As Popup | chromeless/modal"),
 *   field_types = {
 *     "link"
 *   }
 * )
 */
class LinkAsPopupFieldWidget extends LinkWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'available_targets' => [],
      'data-width' => 980,
      'data-height' => 800,
      'data-title' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    $element['available_targets'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t("Available Targets"),
      '#description' => $this->t('The enabled targets that are displayed in the field widget target selection dropdown. If none are selected, all will be available.'),
      '#default_value' => $this->getSetting('available_targets'),
      '#options' => $this->getTargets(),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $targets_conf = $this->getSelectedOptions();
    if ($targets_conf) {
      // Clear out any zeros saved for unchecked values.
      foreach ($targets_conf as $key => $val) {
        if (!$val) {
          unset($targets_conf[$key]);
        }
      }
      $summary[] = !empty($targets_conf) ? $this->t('Available targets:') . ' ' . implode(', ', $targets_conf) : $this->t('No target options were selected.');
    }
    else {
      $summary[] = $this->t('All targets will be available.');
    }
    return $summary;
  }

  /**
   * Helper function to provide an array of target options.
   *
   * @return array
   *   The options to use as targets
   */
  public static function getTargets() {
    return [
      '_self' => t('Current window (_self)'),
      '_blank' => t('New window (_blank)'),
      '_parent' => t('Parent window (_parent)'),
      '_top' => t('Topmost window (_top)'),
      'dialog' => t('Dialog'),
      'chromeless' => t('Chromeless Window'),
      'drupal_ajax' => t('Drupal Ajax [Site pages (Experimental)]'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $field_name = $this->fieldDefinition->getName();
    $route_name = \Drupal::routeMatch()->getRouteName();
    if ( str_starts_with($route_name, 'layout_builder' )===TRUE){
      $field_name = "settings[block_form][$field_name]";
    }
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $item = $this->getLinkItem($items, $delta);
    $options = $item->get('options')->getValue();
    $targets_available = $this->getSelectedOptions(TRUE);
    $defaultSettings = $this->defaultSettings();
    $default_value = !empty($options['attributes']['target']) ? $options['attributes']['target'] : '';
    $element['options']['attributes']['target'] = [
      '#type' => 'select',
      '#title' => $this->t('Select a target'),
      '#options' => ['' => $this->t('- None -')] + $targets_available,
      '#default_value' => $default_value,
    ];
    $element['options']['attributes']['data-width'] = [
      '#type' => 'number',
      '#title' => $this->t('Target Width'),
      '#field_suffix' => $this->t('<abbr title="Pixels">px</abbr>'),
      '#default_value' => isset($options['attributes']['data-width']) ? $options['attributes']['data-width'] : $defaultSettings['data-width'],
      '#min' => 50,
      '#size' => 3,
      '#states' => [
        'visible' => [
          ':input[name="'. $field_name.'[' . $delta . '][options][attributes][target]"]' => [['value' => 'dialog'], ['value' => 'chromeless'], ['value' => 'drupal_ajax']],
        ],
      ],
      '#wrapper_attributes' => [
        'class' => ['col col-6 width'],
      ],
      '#prefix' => '<div class="row popup-size">',
    ];
    $element['options']['attributes']['data-height'] = [
      '#type' => 'number',
      '#title' => $this->t('Target Height'),
      '#field_suffix' => $this->t('<abbr title="Pixels">px</abbr>'),
      '#default_value' => isset($options['attributes']['data-height']) ? $options['attributes']['data-height'] : $defaultSettings['data-height'],
      '#min' => 50,
      '#size' => 3,
      '#states' => [
        'visible' => [
          ':input[name="'. $field_name.'[' . $delta . '][options][attributes][target]"]' => [['value' => 'dialog'], ['value' => 'chromeless'], ['value' => 'drupal_ajax']],
        ],
      ],
      '#wrapper_attributes' => [
        'class' => ['col col-6 height'],
      ],
      '#suffix' => '</div>',
    ];
    //  Reminder #state drupal.ajax will pick up page title.
    $element['options']['attributes']['data-title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Target Window Title'),
      '#size' => 24,
      '#maxlength' => 512,
      '#default_value' => isset($options['attributes']['data-title']) ? $options['attributes']['data-title'] : $defaultSettings['data-title'],
      '#states' => [
        'visible' => [
          ':input[name="'. $field_name.'[' . $delta . '][options][attributes][target]"]' => [['value' => 'dialog'], ['value' => 'chromeless']],
        ],
      ],
      '#wrapper_attributes' => [
        'class' => ['col col-12 title clearfix'],
      ],
      '#suffix' => '</div>',
      '#prefix' => '<div class="row popup-title">',
    ];
    $element['#attached']['library'][] = 'link_as_popup/widget';
    return $element;
  }

  /**
   * Getting link items.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   Returning of field items.
   * @param string $delta
   *   Returning field delta with item.
   *
   * @return \Drupal\link\LinkItemInterface
   *   Returning link items inteface.
   */
  private function getLinkItem(FieldItemListInterface $items, $delta) {
    return $items[$delta];
  }

  /**
   * Retrieve settings and convert to an array that includes on those selected.
   *
   * return array
   *   The options to include.
   */
  public function getSelectedOptions($default_all = FALSE) {
    $defaults = $this->getTargets();
    $targets_available = [];

    // If a subset of targets have been specified, use them.
    $targets_conf = $this->getSetting('available_targets');
    if ($targets_conf) {
      foreach ($targets_conf as $key) {
        if (isset($defaults[$key])) {
          $targets_available[$key] = $defaults[$key];
        }
      }
    }
    if (empty($targets_available) && $default_all) {
      $targets_available = $defaults;
    }
    return $targets_available;
  }

}
